html, body {
    padding: 0px;
    margin: 0px;
    border: none;
    _overflow: hidden;
 }
#presenceToggle {
    border: 1px solid black;
    position: absolute;
    right: 1px;
    top: 1px;
    padding: 2px;
    background: #005522;
    color: white;
    font-weight: bold;
    cursor: pointer;
}
#presenceToggle:hover {
}
#presence {
    float: right;
    border: 1px solid gray;
    width: 130px;
    height: 83%;
    margin-left: 3px;
    margin-bottom: 0px;
    overflow: hidden;
}
#presenceTitle {
    text-decoration: underline;
}
#presenceList {
    overflow: auto;
    height: 90%;
}
#history {
    border: 1px solid gray;
    height: 83%;
    overflow: auto;
    margin-bottom: 0px;
}
#historySpacer {
    border: 1px solid red;    
}
#inputBar {
    margin-top: 0px;
    height: 15%;
}
#inputBar textarea {
    width: 100%;
    height: 100%;
    border: 1px solid gray;
}



#popup {
    position: absolute; 
    height: 100%; 
    width: 70%; 
    left: 15%; 
    z-index: 2;
}

#popupOuter { 
    height: 100%;
    width: 100%;
    overflow: hidden;
    position: relative;
}
#popupOuter[id] {display: table; position: static;}

#popupMiddle {position: absolute; top: 50%;} /* for explorer only*/
#popupMiddle[id] {display: table-cell; vertical-align: middle; position: static;}

#popupContent {position: relative; top: -50%} /* for explorer only */
/* optional: #popupContent[id] {position: static;} */

#popupContent {
    font-size: 120%;
    text-align:center;
    background: #005522;
    color: white;
    border: 5pt solid #50A572;
    padding: 8px;
    padding-bottom: 20px;
    width: 100%;
}
#popupContent p {
    font-size: 150%;
}

#absorbClicks {
    position:absolute;
    top:0;
    left:0;
    width:100%;
    height:100%;
}


#history div {
  background-color:#eeffee;
  padding:.2em;
  padding-left:1.5em;
  text-indent: -1em;
  border-bottom: 1px dotted #ccc;
}
#history .message .user {
  width:5em;
  font-weight: bold;
  color:#DD7200;
  padding-right:.2em;
  text-align:right;
}
#history .message.self .user{
  color: #B42600;
}
#history .message.mentioned{
  background-color: #d5c5ff;
}
#history .message.private{
  background-color: #ccc;
}
#history .message.ctcp {
  background-color: #f0f;
}
#history .action {
  color: #D68850;
  font-style:italic;
}
#history .action {
  color: #A94C3B;
  font-style:italic;
}
#history .informative {
  font-size: 85%;
  font-style: italic;
  color:#999;
  text-align:center;
  padding:.3em;
}
#presenceList .user_entry {
  font-size: 90%;
  padding:.2em;
  padding-left:8pt;
  border-bottom: 1px dotted #ccc;
}



